@@ -37,6 +37,7 @@
 #include "conv_table.h"
 #include "scim_skk_intl.h"
 #include "scim_skk_config.h"
+#include "scim_skk_history.h"
 
 using namespace scim_skk;
 
@@ -62,7 +63,7 @@ using namespace scim_skk;
 
 static ConfigPointer  _scim_config (0);
 static SKKDictionary *scim_skkdict = 0;
-
+static History        scim_skkhistory;
 
 
 extern "C" {
@@ -115,7 +116,6 @@ SKKFactory::SKKFactory (const String &lang,
                         const String &uuid,
                         const ConfigPointer &config)
     :  m_uuid(uuid),
-       m_sysdictpath(SCIM_SKK_CONFIG_SYSDICT_DEFAULT),
        m_userdictname(SCIM_SKK_CONFIG_USERDICT_DEFAULT),
        m_config(config)
 {
@@ -157,7 +157,7 @@ SKKFactory::get_icon_file () const
 WideString
 SKKFactory::get_authors () const
 {
-    return utf8_mbstowcs("Jun Mukai");
+    return utf8_mbstowcs("(C) 2004-2005 Jun Mukai <mukai@jmuk.org>");
 }
 
 WideString
@@ -169,7 +169,9 @@ SKKFactory::get_credits () const
 WideString
 SKKFactory::get_help () const
 {
-    return WideString();
+    return utf8_mbstowcs(
+#include "scim_skk_help_message.txt"
+);
 }
 
 void
@@ -189,13 +191,23 @@ SKKFactory::reload_config (const ConfigPointer &config)
 {
     if (config) {
         String str;
+        std::vector<String> sysdicts_default;
 
-        m_sysdictpath = config->read(String(SCIM_SKK_CONFIG_SYSDICT),
-                                     String(SCIM_SKK_CONFIG_SYSDICT_DEFAULT));
-        scim_skkdict->add_sysdict(m_sysdictpath);
+        scim_split_string_list (sysdicts_default,
+                                String (SCIM_SKK_CONFIG_SYSDICT_DEFAULT));
+        m_sysdicts = config->read(String(SCIM_SKK_CONFIG_SYSDICT),
+                                  sysdicts_default);
+        if (m_sysdicts.size() > 0) {
+            for (std::vector<String>::const_iterator it = m_sysdicts.begin();
+                 it != m_sysdicts.end(); it++) {
+                scim_skkdict->add_sysdict(*it);
+            }
+        } else {
+            scim_skkdict->add_sysdict(String(SCIM_SKK_CONFIG_SYSDICT_DEFAULT));
+        }
         m_userdictname = config->read(String(SCIM_SKK_CONFIG_USERDICT),
                                       String(SCIM_SKK_CONFIG_USERDICT_DEFAULT));
-        scim_skkdict->set_userdict(m_userdictname);
+        scim_skkdict->set_userdict(m_userdictname, scim_skkhistory);
         candvec_size =
             config->read(String(SCIM_SKK_CONFIG_CANDVEC_SIZE),
                          SCIM_SKK_CONFIG_CANDVEC_SIZE_DEFAULT);
@@ -271,6 +283,12 @@ SKKFactory::reload_config (const ConfigPointer &config)
         str = config->read(String(SCIM_SKK_CONFIG_UPCASE_KEY),
                            String(SCIM_SKK_CONFIG_UPCASE_KEY_DEFAULT));
         m_keybind.set_upcase_keys(str);
+        str = config->read(String(SCIM_SKK_CONFIG_COMPLETION_KEY),
+                           String(SCIM_SKK_CONFIG_COMPLETION_KEY_DEFAULT));
+        m_keybind.set_completion_keys(str);
+        str = config->read(String(SCIM_SKK_CONFIG_COMPLETION_BACK_KEY),
+                           String(SCIM_SKK_CONFIG_COMPLETION_BACK_KEY_DEFAULT));
+        m_keybind.set_completion_back_keys(str);
         str = config->read(String(SCIM_SKK_CONFIG_SELECTION_STYLE),
                            String(SCIM_SKK_CONFIG_SELECTION_STYLE_DEFAULT));
         m_keybind.set_selection_style(str);
@@ -284,7 +302,8 @@ SKKInstance::SKKInstance (SKKFactory   *factory,
                           int           id)
     : IMEngineInstanceBase (factory, encoding, id),
       m_skk_mode (SKK_MODE_HIRAGANA),
-      m_skkcore (&(factory->m_keybind), &(m_key2kana), scim_skkdict)
+      m_skkcore (&(factory->m_keybind), &(m_key2kana),
+                 scim_skkdict, scim_skkhistory)
 {
     SCIM_DEBUG_IMENGINE(1) << "Create SKK Instance : ";
     init_key2kana();
