@@ -20,7 +20,7 @@
 
 #include <gtk/gtk.h>
 #include <vector>
-
+#include <algorithm>
 
 #include <scim.h>
 
@@ -31,15 +31,17 @@
 #include "scim_skk_intl.h"
 
 using namespace scim;
+using namespace std;
 
 namespace scim_skk {
 
 extern bool __have_changed;
 extern GtkWidget *__widget_sysdicts;
+extern vector<String> __config_sysdicts;
 
 enum DictColumnType {
-    DICT_TYPE_COLUMN, /* string: type of dictionary */
-    DICT_NAME_COLUMN, /* string: dictionary name */
+    DICT_TYPE_COLUMN,  /* string: type of dictionary */
+    DICT_NAME_COLUMN,  /* string: dictionary name */
     DICT_N_COLUMNS
 };
 
@@ -60,21 +62,21 @@ static String __dict_type_names [] = {
 
 static DictionaryConfigWidgets __widgets_dicts [] = {
     {
-        N_("System Dictionary Path:"),
+        _("System Dictionary Path:"),
         NULL,
         NULL,
         NULL,
         NULL
     },
     {
-        N_(""),
+        "",
         NULL,
         NULL,
         NULL,
         NULL
     },
     {
-        N_("CDB Dictionary Path:"),
+        _("CDB Dictionary Path:"),
         NULL,
         NULL,
         NULL,
@@ -91,6 +93,8 @@ static DictionaryConfigWidgets __widgets_dicts [] = {
 
 static GtkWidget *__combo_box_dict_types = NULL;
 
+static GtkListStore *__dict_list_store = NULL;
+
 
 static void
 file_selection_clicked_cb (GtkButton *button,
@@ -155,11 +159,16 @@ dict_list_add_clicked_cb (GtkButton *button,
             }
         }
     }
+    
+    __config_sysdicts.push_back(dict_type+String(":")+dict_name);
+
     gtk_list_store_append(GTK_LIST_STORE(model), &iter);
     gtk_list_store_set(GTK_LIST_STORE(model), &iter,
                        DICT_TYPE_COLUMN, dict_type.data(),
                        DICT_NAME_COLUMN, dict_name.data(),
                        -1);
+
+    __have_changed = true;
 }
 
 static void
@@ -171,7 +180,13 @@ dict_list_delete_clicked_cb (GtkButton *button,
     GtkTreeSelection *selection = gtk_tree_view_get_selection(view);
     GtkTreeIter iter;
     if (gtk_tree_selection_get_selected(selection, NULL, &iter)) {
+        gint i;
+        GtkTreePath *path = gtk_tree_model_get_path(model, &iter);
+        i = gtk_tree_path_get_indices(path)[0];
         gtk_list_store_remove(GTK_LIST_STORE(model), &iter);
+
+        __config_sysdicts.erase(__config_sysdicts.begin()+i);
+        __have_changed = true;
     }
 }
 
@@ -186,7 +201,15 @@ dict_list_up_clicked_cb (GtkButton *button,
     if (gtk_tree_selection_get_selected(selection, NULL, &iter)) {
         GtkTreePath *path;
         GtkTreeIter prev_iter;
+        int i;
         path = gtk_tree_model_get_path(model, &iter);
+        i = gtk_tree_path_get_indices(path)[0];
+        if (i != 0) {
+            vector<String>::iterator it  = __config_sysdicts.begin() + i;
+            vector<String>::iterator it2 = it - 1;
+            iter_swap(it, it2);
+            __have_changed = true;
+        }
         if (gtk_tree_path_prev(path) &&
             gtk_tree_model_get_iter(model, &prev_iter, path)) {
             gtk_list_store_move_before(GTK_LIST_STORE(model),
@@ -207,7 +230,16 @@ dict_list_down_clicked_cb (GtkButton *button,
     if (gtk_tree_selection_get_selected(selection, NULL, &iter)) {
         GtkTreePath *path;
         GtkTreeIter next_iter;
+        int i;
         path = gtk_tree_model_get_path(model, &iter);
+        i = gtk_tree_path_get_indices(path)[0];
+        if (i < __config_sysdicts.size() - 1) {
+            vector<String>::iterator it  = __config_sysdicts.begin() + i;
+            vector<String>::iterator it2 = it + 1;
+            iter_swap(it, it2);
+            __have_changed = true;
+        }
+
         gtk_tree_path_next(path);
         if (gtk_tree_model_get_iter(model, &next_iter, path)) {
             gtk_list_store_move_after(GTK_LIST_STORE(model),
@@ -260,27 +292,31 @@ dict_list_selection_changed_cb (GtkTreeSelection *selection,
     }
 }
 
-inline GtkListStore*
-dict_list_setup (std::vector<String> &data)
+GtkListStore*
+dict_list_setup (vector<String> &data)
 {
-    GtkListStore *retval =
-        gtk_list_store_new(DICT_N_COLUMNS,
-                           G_TYPE_STRING,  /* dict type */
-                           G_TYPE_STRING); /* dict name */
+    if (__dict_list_store) {
+        gtk_list_store_clear(__dict_list_store);
+    } else {
+        __dict_list_store = 
+            gtk_list_store_new(DICT_N_COLUMNS,
+                               G_TYPE_STRING,  /* dict type */
+                               G_TYPE_STRING); /* dict name */
+    }
 
     for (int i = 0; i < data.size(); i++) {
         GtkTreeIter treeiter;
         int colon_pos;
-        gtk_list_store_append(retval, &treeiter);
+        gtk_list_store_append(__dict_list_store, &treeiter);
         colon_pos = data[i].find(':');
-        gtk_list_store_set(retval, &treeiter,
+        gtk_list_store_set(__dict_list_store, &treeiter,
                            DICT_TYPE_COLUMN,
                            (colon_pos == String::npos)? "DictFile" : data[i].substr(0, colon_pos).data(),
                            DICT_NAME_COLUMN,
                            (colon_pos == String::npos)? data[i].data() : data[i].substr(colon_pos+1, String::npos).data(),
                            -1);
     }
-    return retval;
+    return __dict_list_store;
 }
 
 inline GtkWidget*
@@ -294,13 +330,13 @@ dict_list_model_setup (GtkTreeModel *model)
 
     renderer = gtk_cell_renderer_text_new();
     column =
-        gtk_tree_view_column_new_with_attributes("type",
+        gtk_tree_view_column_new_with_attributes(_("type"),
                                                  renderer,
                                                  "text", DICT_TYPE_COLUMN,
                                                  NULL);
     gtk_tree_view_append_column(GTK_TREE_VIEW(retval), column);
     column =
-        gtk_tree_view_column_new_with_attributes("name",
+        gtk_tree_view_column_new_with_attributes(_("dict name"),
                                                  renderer,
                                                  "text", DICT_NAME_COLUMN,
                                                  NULL);
@@ -376,7 +412,7 @@ dict_entry_widgets_setup (GtkBox *container,
 
     hbox = gtk_hbox_new(FALSE, 0);
     gtk_widget_show(hbox);
-    label = gtk_label_new("Dictionary Type: ");
+    label = gtk_label_new(_("Dictionary Type: "));
     gtk_widget_show(label);
     gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 4);
     __combo_box_dict_types = gtk_combo_box_new_text();
@@ -423,98 +459,50 @@ dict_entry_widgets_setup (GtkBox *container,
     gtk_box_pack_start(container, hbox, FALSE, FALSE, 4);
 }
 
-void
-on_default_dict_selection_clicked (GtkButton *button,
-                                   gpointer   user_data)
+GtkWidget*
+dict_selection_widget_setup (void)
 {
-    std::vector<String> *data = static_cast<std::vector<String>*>(user_data);
-    if (data) {
-        GtkWidget *dialog = GTK_WIDGET(gtk_dialog_new());
-        GtkWidget *ok_button, *cancel_button;
-        gint result;
-        GtkWidget *tree;
-        GtkListStore *__dict_list_store;
+    GtkWidget *dialog = GTK_WIDGET(gtk_dialog_new());
+    GtkWidget *ok_button, *cancel_button;
+    //GtkWidget *container = gtk_vbox_new(FALSE, 0);
+    GtkWidget *tree;
 
-        gtk_window_set_title(GTK_WINDOW(dialog),
-                             _("Configure Dictionaries"));
-        __dict_list_store = dict_list_setup(*data);
+    __widget_sysdicts = gtk_vbox_new(FALSE, 0);
+    dict_list_setup(__config_sysdicts);
 
-        /* setup dialog for dictionaries selection */
-        gtk_container_set_border_width(GTK_CONTAINER(dialog), 4);
-        gtk_window_set_resizable(GTK_WINDOW(dialog), TRUE);
+    /* setup container for dictionaries selection */
+    gtk_container_set_border_width(GTK_CONTAINER(__widget_sysdicts), 4);
 
-        tree = dict_list_model_setup(GTK_TREE_MODEL(__dict_list_store));
+    tree = dict_list_model_setup(GTK_TREE_MODEL(__dict_list_store));
 
-        gtk_widget_show(tree);
+    gtk_widget_show(tree);
 
-        {
-            GtkWidget *hbox, *vbox, *button;
-            hbox = gtk_hbox_new(FALSE, 0);
-            gtk_widget_show(hbox);
-            gtk_box_pack_start(GTK_BOX(hbox), tree, FALSE, FALSE, 4);
-            vbox = gtk_vbox_new(FALSE, 0);
-            button = gtk_button_new_from_stock(GTK_STOCK_GO_UP);
-            gtk_box_pack_start(GTK_BOX(vbox), button, TRUE, FALSE, 4);
-            g_signal_connect((gpointer) button, "clicked",
-                             G_CALLBACK(dict_list_up_clicked_cb),
-                             (gpointer) tree);
-            button = gtk_button_new_from_stock(GTK_STOCK_GO_DOWN);
-            gtk_box_pack_start(GTK_BOX(vbox), button, TRUE, FALSE, 4);
-            g_signal_connect((gpointer) button, "clicked",
-                             G_CALLBACK(dict_list_down_clicked_cb),
-                             (gpointer) tree);
-            gtk_widget_show_all(vbox);
-            gtk_box_pack_start(GTK_BOX(hbox), vbox, FALSE, FALSE, 4);
-            gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), hbox,
-                               FALSE, FALSE, 4);
-        }
-
-        dict_entry_widgets_setup(GTK_BOX(GTK_DIALOG(dialog)->vbox), GTK_TREE_VIEW(tree));
-
-        cancel_button = gtk_dialog_add_button(GTK_DIALOG(dialog),
-                                              GTK_STOCK_CANCEL,
-                                              GTK_RESPONSE_CANCEL);
-        ok_button = gtk_dialog_add_button(GTK_DIALOG(dialog),
-                                          GTK_STOCK_OK,
-                                          GTK_RESPONSE_OK);
-        gtk_widget_grab_default(ok_button);
-        gtk_dialog_set_has_separator(GTK_DIALOG(dialog), TRUE);
-        gtk_widget_show(dialog);
-
-        result = gtk_dialog_run(GTK_DIALOG(dialog));
-        if (result == GTK_RESPONSE_OK) {
-            GtkTreeIter iter;
-            gtk_tree_model_get_iter_first(GTK_TREE_MODEL(__dict_list_store),
-                                          &iter);
-            data->clear();
-            do {
-                String s;
-                gchar *dict_type, *dict_name;
-                gtk_tree_model_get(GTK_TREE_MODEL(__dict_list_store), &iter,
-                                   DICT_TYPE_COLUMN, &dict_type,
-                                   DICT_NAME_COLUMN, &dict_name,
-                                   -1);
-                s.assign(String(dict_type));
-                s.append(1, ':');
-                s.append(String(dict_name));
-                data->push_back(s);
-            } while (gtk_tree_model_iter_next(GTK_TREE_MODEL(__dict_list_store), &iter));
-
-            if (data->size()>0) {
-                std::vector<String>::iterator it = data->begin();
-                String s = *it;
-                for (; it != data->end(); it++) {
-                    s.append(1, ',');
-                    s.append(*it);
-                }
-                gtk_entry_set_text(GTK_ENTRY(__widget_sysdicts), s.data());
-            }
-            __have_changed = true;
-        }
-
-        gtk_widget_destroy (dialog);
+    {
+        GtkWidget *hbox, *vbox, *button;
+        hbox = gtk_hbox_new(FALSE, 0);
+        gtk_widget_show(hbox);
+        gtk_box_pack_start(GTK_BOX(hbox), tree, FALSE, FALSE, 4);
+        vbox = gtk_vbox_new(FALSE, 0);
+        button = gtk_button_new_from_stock(GTK_STOCK_GO_UP);
+        gtk_box_pack_start(GTK_BOX(vbox), button, TRUE, FALSE, 4);
+        g_signal_connect((gpointer) button, "clicked",
+                         G_CALLBACK(dict_list_up_clicked_cb),
+                         (gpointer) tree);
+        button = gtk_button_new_from_stock(GTK_STOCK_GO_DOWN);
+        gtk_box_pack_start(GTK_BOX(vbox), button, TRUE, FALSE, 4);
+        g_signal_connect((gpointer) button, "clicked",
+                         G_CALLBACK(dict_list_down_clicked_cb),
+                         (gpointer) tree);
+        gtk_widget_show_all(vbox);
+        gtk_box_pack_start(GTK_BOX(hbox), vbox, FALSE, FALSE, 4);
+        gtk_box_pack_start(GTK_BOX(__widget_sysdicts), hbox, FALSE, FALSE, 4);
     }
+
+    dict_entry_widgets_setup(GTK_BOX(__widget_sysdicts), GTK_TREE_VIEW(tree));
+
+    gtk_widget_show(__widget_sysdicts);
+
+    return __widget_sysdicts;
 }
 
-
 }
